<?php 

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

/**
 * Settings Page General Tab
 * 
 * The code for the plugins settings page general tab
 * 
 * @package Follow My Blog Post
 * @since 1.0.0
 */

//get all pages
$get_pages = get_pages();

?>
<!-- beginning of the general settings meta box -->
<div id="wpw-fp-general" class="post-box-container">
	<div class="metabox-holder">
		<div class="meta-box-sortables ui-sortable">
			<div id="general" class="postbox">
				<div class="handlediv" title="<?php esc_html_e( 'Click to toggle', 'wpwfp' ); ?>"><br /></div>
					
					<!-- general settings box title -->
					<h3 class="hndle">
						<span style='vertical-align: top;'><?php esc_html_e( 'General Settings', 'wpwfp' ); ?></span>
					</h3>

					<div class="inside">
					
					<table class="form-table">
						<tbody>
						
							<tr>
								<td colspan="2">
									<?php
										echo apply_filters ( 'wpweb_fb_settings_submit_button', '<input class="button-primary wpw-fp-save-btn" type="submit" name="wpw-fp-set-submit" value="'.esc_html__( 'Save Changes','wpwfp' ).'" />' );
									?>
								</td>
							</tr>
							
							<?php
							// do action for add setting before general settings
							do_action( 'wpw_fp_before_general_setting', $wpw_fp_options );
							?>
							
							<tr valign="top">
								<th scope="row">
									<label for="wpw_fp_options[del_all_options]"><?php esc_html_e( 'Delete Options', 'wpwfp' ); ?></label>
								</th>
								<td>
									<input type="checkbox" name="wpw_fp_options[del_all_options]" id="wpw_fp_options[del_all_options]" value="1" <?php checked( '1', isset( $wpw_fp_options['del_all_options'] ) ? $wpw_fp_options['del_all_options'] : '' ) ?> /><br />
									<span class="description"><?php esc_html_e( 'If you don\'t want to use the Follow My Blog Post Plugin on your site anymore, you can check that box. This makes sure, that all the settings and tables are being deleted from the database when you deactivate the plugin.', 'wpwfp' ); ?></span>
								</td>
							</tr>
							
							<tr>
								<th scope="row">
									<label for="wpw_fp_options[disable_follow_guest]"><?php esc_html_e('Disable Guest Followers', 'wpwfp');?></label>
								</th>
								<td>
									<input type="checkbox" id="wpw_fp_options[disable_follow_guest]" name="wpw_fp_options[disable_follow_guest]" value="1" <?php checked( '1', isset( $wpw_fp_options['disable_follow_guest'] ) ? $wpw_fp_options['disable_follow_guest'] : '' ) ?>/><br />
									<span class="description"><?php esc_html_e( 'Guests (non-logged-in users) have the permission to follow any post by default. If you check this option, then Follow button would not be displayed for Guest users.', 'wpwfp' ); ?></span>
								</td>
							</tr>
							
							<tr valign="top">
								<th scope="row">
									<label for="wpw_fp_options[enable_log]"><?php esc_html_e( 'Enable Email Logs', 'wpwfp' ); ?></label>
								</th>
								<td>
									<input type="checkbox" name="wpw_fp_options[enable_log]" id="wpw_fp_options[enable_log]" value="1" <?php checked( '1', isset( $wpw_fp_options['enable_log'] ) ? $wpw_fp_options['enable_log'] : '' ) ?> /><br />
									<span class="description"><?php printf( esc_html__( 'Check this box if you want to log all emails into database. Enabling this will add a new column on followers listing page to view the log entries.', 'wpwfp' ).'<br><strong>'.esc_html__('Note', 'wpwfp') .': </strong>'.esc_html__( 'It is', 'wpwfp').'<b> '.esc_html__( 'not recommended', 'wpwfp').'</b> '.esc_html__( 'to enable this option, as this will slow down the plugin.', 'wpwfp' ) ); ?></span>
								</td>
							</tr>
							
							<?php
							// do action for add setting after enable email settings
							do_action( 'wpw_fp_after_enable_email_log_setting', $wpw_fp_options );
							?>
							
							<tr valign="top">
								<th scope="row">
									<label for="wpw_fp_options[disable_follow_post_meta_box]"><?php esc_html_e( 'Disable Follow My Blog Post Settings Meta Box', 'wpwfp' ); ?></label>
								</th>
								<td>
									<ul>
									<?php 
										$all_types = get_post_types( array( 'public' => true ), 'objects' );
										$all_types = is_array( $all_types ) ? $all_types : array();
										
										if( !empty( $wpw_fp_options['disable_follow_post_meta_box'] ) ) {
											$disable_meta_box = $wpw_fp_options['disable_follow_post_meta_box'];
										} else {
											$disable_meta_box = array();
										}
										
										foreach( $all_types as $type ) {	
											
											if( !is_object( $type ) ) 
												continue;															
												
											$label = $type->labels->name ? $type->labels->name : $type->name;
											
											if( $label == 'Media' || $label == 'media' ) 
												continue; // skip media
												
											$selected = ( in_array( $type->name, $disable_meta_box ) ) ? 'checked="checked"' : '';													
										?>
															
										<li class="wpwfp-disable-meta-box">
											<input type="checkbox" id="wpwfp_disable_meta_box_<?php echo $type->name; ?>" name="wpw_fp_options[disable_follow_post_meta_box][]" value="<?php echo $type->name; ?>" <?php echo $selected; ?>/>
											<label for="wpwfp_disable_meta_box_<?php echo $type->name; ?>"><?php echo $label; ?></label>
										</li>
										<?php } ?>
									</ul>
									<span class="description"><?php esc_html_e( 'If you check this box, Follow My Blog Post Settings Meta Box will not be displayed on selected post types.', 'wpwfp' ); ?></span>  
								</td>
							</tr>

							<tr valign="top">
								<th scope="row">
									<label for="wpw_fp_options[enable_notify_followers]"><?php esc_html_e( 'Auto Enable Notify Followers', 'wpwfp' ); ?></label>
								</th>
								<td>
									<input type="checkbox" name="wpw_fp_options[enable_notify_followers]" id="wpw_fp_options[enable_notify_followers]" value="1" <?php checked( '1', isset( $wpw_fp_options['enable_notify_followers'] ) ? $wpw_fp_options['enable_notify_followers'] : '' ) ?> /><br />
									<span class="description"><?php printf( esc_html__( 'Check this box to automatically enable "Notify followers for this update" for all Post / Pages / Custom post types.', 'wpwfp' )); ?></span>
								</td>
							</tr>

							<!-- Posts Follow Settings Start -->
							<tr>
								<th colspan="2">
									<h3><?php esc_html_e( 'Posts Follow Settings', 'wpwfp' ); ?></h3>
								</th>
							</tr>
							
							<tr valign="top">
								<th scope="row">
									<label for="wpw_fp_options[follow_buttons]"><?php esc_html_e( 'Follow Button Text', 'wpwfp' ); ?></label>
								</th>
								<td>
									<input type="text" name="wpw_fp_options[follow_buttons][follow]" value="<?php echo $wpw_fp_model->wpw_fp_escape_attr( $wpw_fp_options['follow_buttons']['follow'] ); ?>" placeholder="<?php esc_html_e( 'Follow', 'wpwfp' );?>"/>
									<input type="text" name="wpw_fp_options[follow_buttons][following]" value="<?php echo $wpw_fp_model->wpw_fp_escape_attr( $wpw_fp_options['follow_buttons']['following'] ); ?>" placeholder="<?php esc_html_e( 'Following', 'wpwfp' );?>"/>
									<input type="text" name="wpw_fp_options[follow_buttons][unfollow]" value="<?php echo $wpw_fp_model->wpw_fp_escape_attr( $wpw_fp_options['follow_buttons']['unfollow'] ); ?>" placeholder="<?php esc_html_e( 'UnFollow', 'wpwfp' );?>"/></br>
									<span class="description"><?php esc_html_e( 'Enter Follow, Following and Unfollow button text respectively. Default button text are Follow, Following and Unfollow respectively.', 'wpwfp' ); ?></span>
								</td>
							</tr>
							
							<tr valign="top">
								<th scope="row">
									<label for="wpw_fp_options[follow_message]"><?php esc_html_e( 'Followers Counter Message', 'wpwfp' ); ?></label>
								</th>
								<td>
									<input type="text" name="wpw_fp_options[follow_message]" id="follow_message" value="<?php echo $wpw_fp_model->wpw_fp_escape_attr( $wpw_fp_options['follow_message'] ); ?>" size="76" /></br>
									<span class="description"><?php printf( esc_html__( 'Here you can give the custom message to show followers count for Post / Page. Leave it blank to hide the followers counter. %s - displays the followers count.', 'wpwfp' ), '<br /><code>{followers_count}</code>'); ?></span>
								</td>
							</tr>
							
							<tr valign="top">
								<th scope="row">
									<label for="wpw_fp_options[follow_form_location]"><?php esc_html_e( 'Follow Button Location', 'wpwfp' ); ?></label>
								</th>
								<td>
									<select name="wpw_fp_options[follow_form_location]" id="wpw_fp_options[follow_form_location]" class="chosen-select">
										<?php   												
											$select_location = array( 'after_content' => esc_html__( 'After Content', 'wpwfp' ), 'before_content' => esc_html__( 'Before Content', 'wpwfp' ) );
															
											foreach ( $select_location as $key => $option ) {											
												?>
												<option value="<?php echo $key; ?>" <?php selected( $wpw_fp_options['follow_form_location'], $key ); ?>>
													<?php echo $option; ?>
												</option>
												<?php
											}														
										?> 														
									</select><br />
									<span class="description"><?php esc_html_e( 'Select the location where you want the follow button being displayed.', 'wpwfp' ); ?></span>
								</td>
							</tr>
							
							<tr valign="top">
								<th scope="row">
									<label for="wpw_fp_options[follow_form_position]"><?php esc_html_e( 'Follow Button Position', 'wpwfp' ); ?></label>
								</th>
								<td>
									<select name="wpw_fp_options[follow_form_position]" id="wpw_fp_options[follow_form_position]" class="chosen-select">
										<?php 
											$select_location = array( 'left' => esc_html__( 'Left', 'wpwfp' ), 'right' => esc_html__( 'Right', 'wpwfp' ) );
											
											foreach ( $select_location as $key => $option ) { 
												?>
												<option value="<?php echo $key; ?>" <?php selected( $wpw_fp_options['follow_form_position'], $key ); ?>>
													<?php echo $option; ?>
												</option><?php
											}
										?>
									</select> <br />
									<span class="description"><?php esc_html_e( 'Select the position where you want the follow button being displayed.', 'wpwfp' ); ?></span>
								</td>
							</tr>
							
							<tr valign="top">
								<th scope="row">
									<label for="wpw_fp_options[prevent_type]"><?php esc_html_e( 'Enable Follow Button On', 'wpwfp' ); ?></label>
								</th>
								<td>
									<ul class="wpw_fp_indvisual" id="wpw_fp_indvisual">
									<?php 
										$all_types = get_post_types( array( 'public' => true ), 'objects');
										$all_types = is_array( $all_types ) ? $all_types : array();
										unset( $all_types['attachment'] );
										
										if( !empty( $wpw_fp_options['prevent_type'] ) ) {
											$prevent_types = $wpw_fp_options['prevent_type'];
										} else {
											$prevent_types = '';
										}
										
										$prevent_types = is_array( $prevent_types ) ? $prevent_types : array();
										
										foreach ( $all_types as $type ) {
											if( !empty( $wpw_fp_options['prevent_item_'.$type->name] ) ) {
												$items_count = count( $wpw_fp_options['prevent_item_'.$type->name] );
											} else {
												$wpw_fp_options['prevent_item_'.$type->name] = '';
												$items_count = 0;
											}
											
											if ( !is_object( $type ) ) continue;
												$label = $type->labels->name ? $type->labels->name : $type->name;
												$selected = ( in_array( $type->name, $prevent_types ) ) ? 'checked="checked"' : '';
												
												$individual = '<a href="javascript:void(0);" class="wpw_fp_indivisual_entries" onclick="wpw_fp_popup_indivisual( \'' . $type->name . '\', \'\' )">' . esc_html__( 'Choose individual entries', 'wpwfp' ) . '</a>';
									?>
										
										<li>
											<input type="checkbox" id="wpw_fp_prevent_<?php echo $type->name; ?>" name="wpw_fp_options[prevent_type][]" value="<?php echo $type->name; ?>" <?php echo $selected; ?>/>
											
											<label for="wpw_fp_prevent_<?php echo $type->name; ?>"><?php echo $label; ?></label>
												<?php echo $individual; ?>&nbsp;&nbsp;&nbsp;<label id="wpw_fp_selected_indivisual_<?php echo $type->name; ?>"><?php echo '( ' . $items_count . ' ) ';?><?php esc_html_e( 'Selected', 'wpwfp' ); ?></label>
										</li>
										
									<?php 	$args = array( 'post_type' => $type->name, 'posts_per_page' => '-1' );
											$wp_query = null;
											$wp_query = new WP_Query;
											$wp_query->query( $args );
											
											if( !empty( $wpw_fp_options['prevent_item_'.$type->name] ) ) {
												$prevent_items = $wpw_fp_options['prevent_item_'.$type->name];
											} else {
												$prevent_items = '';
											}
														
											$prevent_items = is_array( $prevent_items ) ? $prevent_items : array();
									?>
																																		
											<!-- listing all post related post -->
											<div id="fp_indivisual_data_<?php echo $type->name; ?>" class="wpw_fp_indivisual_entries_data" style="display:none;">
												<div class="wpw_fp_indivisual_header">
													<div class="wpw_fp_indivisual_title">
														<h2><?php echo $label; ?></h2>
													</div>
													
													<div class="wpw_fp_indivisual_submit" >

                                                            <a href="javascript:void(0);" class="wpw_fp_checkall_entries" onclick="wpw_fp_toggle_check_uncheck_all('<?php echo $type->name;?>', '', 1);"><?php esc_html_e( 'Check all', 'wpwfp' ); ?></a>&nbsp;&nbsp;&nbsp;<a href="javascript:void(0);" class="wpw_fp_uncheckall_entries" onclick="wpw_fp_toggle_check_uncheck_all('<?php echo $type->name;?>', '', 0);"><?php esc_html_e( 'Uncheck all', 'wpwfp' ); ?></a>&nbsp;&nbsp;&nbsp;															
														<input type="submit" value="<?php esc_html_e( 'Done', 'wpwfp' ); ?>" id="wpw_fp_set_submit_indivisual" class="button-primary">
													</div><!-- .wpw_fp_indivisual_submit -->
												</div><!-- .wpw_fp_indivisual_header -->
												
												<?php 	echo '<ul>';
														
														if ( $wp_query->have_posts() ) {
														while ( $wp_query->have_posts() ) { $wp_query->the_post();

															$checked = ( in_array( $wp_query->post->ID, $prevent_items ) ) ? 'checked="checked"' : '';
															$posttitle = !empty( $wp_query->post->post_title ) ? $wp_query->post->post_title : esc_html__( '(no title)', 'wpwfp' );
																if( strlen( $posttitle ) > 50 ) {
																	$posttitle = substr( $posttitle, 0, 50 );
																	$posttitle = $posttitle.'...';
																} else {
																	$posttitle = $posttitle;
																}?>
															
															<li>
																<input type="checkbox" id="wpw_fp_prevent_<?php echo $wp_query->post->ID; ?>" name="wpw_fp_options[prevent_item_<?php echo $type->name; ?>][]" value="<?php echo $wp_query->post->ID; ?>" <?php echo $checked; ?> />
																
																<label for="wpw_fp_prevent_<?php echo $wp_query->post->ID;?>"><?php echo $posttitle; ?></label>
															</li><?php	
														}
													} else { ?>
														<li><?php esc_attr_e( 'No Entries for this type.', 'wpwfp' ); ?></li><?php	
													} 
													
													wp_reset_query(); 

													echo '</ul>';?>
											</div><!--fp_indivisual_data_--><?php	
										} ?>
									</ul>
									<span class="description"><?php esc_html_e( 'Check each of the post types on which you want the follow button being displayed. You will see a list of all available content types from your site. If you click on the checkbox beside a content type, this means that all content of this type will have the follow button added automatically. Click on "choose individual entries" and a pop up window will appear from which you can select individual posts on which you want to have the follow button being added.', 'wpwfp' ); ?></span>
								</td>
							</tr>

							<tr valign="top">
								<th scope="row">
									<label for="wpw_auto_poster_options[disable_auto_follow_add_comment]"><?php esc_html_e( 'Disable auto follow when comment added', 'wpwfp' ); ?></label>
								</th>
								<td>
									<input id="wpw_fp_options[disable_auto_follow_add_comment]" name="wpw_fp_options[disable_auto_follow_add_comment]" type="checkbox" value="1" <?php if ( isset( $wpw_fp_options['disable_auto_follow_add_comment'] ) ) { checked( '1', $wpw_fp_options['disable_auto_follow_add_comment'] ); } ?> />
									<br />
									<span class="description"><?php esc_html_e( 'Check this box if you want to disable auto follow to post when user add comment. By default when user add comment to post, he will automatically added in post followers list.', 'wpwfp' ); ?></span>
								</td>
							</tr>
							<!-- Posts Follow Settings End -->
							
							<!-- Terms Follow Settings Start -->
							<tr>
								<th colspan="2">
									<h3><?php esc_html_e( 'Terms Follow Settings', 'wpwfp' ); ?></h3>
								</th>
							</tr>
							
							<tr valign="top">
								<th scope="row">
									<label for="wpw_fp_options[term_follow_buttons]"><?php esc_html_e( 'Follow Button Text', 'wpwfp' ); ?></label>
								</th>
								<td>
									<input type="text" name="wpw_fp_options[term_follow_buttons][follow]" value="<?php echo $wpw_fp_model->wpw_fp_escape_attr( $wpw_fp_options['term_follow_buttons']['follow'] ); ?>" placeholder="<?php esc_html_e( 'Follow', 'wpwfp' );?>"/>
									<input type="text" name="wpw_fp_options[term_follow_buttons][following]" value="<?php echo $wpw_fp_model->wpw_fp_escape_attr( $wpw_fp_options['term_follow_buttons']['following'] ); ?>" placeholder="<?php esc_html_e( 'Following', 'wpwfp' );?>"/>
									<input type="text" name="wpw_fp_options[term_follow_buttons][unfollow]" value="<?php echo $wpw_fp_model->wpw_fp_escape_attr( $wpw_fp_options['term_follow_buttons']['unfollow'] ); ?>" placeholder="<?php esc_html_e( 'UnFollow', 'wpwfp' );?>"/></br>
									<span class="description"><?php printf( esc_html__( 'Enter Follow, Following and Unfollow button text respectively. Default button text are Follow, Following and Unfollow respectively. %s - displays the title of the term.', 'wpwfp' ), '<br /><code>{term_name}</code>' ); ?></span>
								</td>
							</tr>
							
							<tr valign="top">
								<th scope="row">
									<label for="wpw_fp_options[term_follow_message]"><?php esc_html_e( 'Followers Counter Message', 'wpwfp' ); ?></label>
								</th>
								<td>
									<input type="text" name="wpw_fp_options[term_follow_message]" id="term_follow_message" value="<?php echo $wpw_fp_model->wpw_fp_escape_attr( $wpw_fp_options['term_follow_message'] ); ?>" size="76" /></br>
									<span class="description"><?php printf( esc_html__( 'Here you can give the custom message to show followers count for Terms. Leave it blank to hide the followers counter.%s - displays the followers count.', 'wpwfp' ), '<br /><code>{followers_count}</code>' ); ?></span>
								</td>
							</tr>
							<!-- Terms Follow Settings End -->
							
							<!-- Author Follow Settings Start -->
							<tr>
								<th colspan="2">
									<h3><?php esc_html_e( 'Authors Follow Settings', 'wpwfp' ); ?></h3>
								</th>
							</tr>
							
							<tr valign="top">
								<th scope="row">
									<label for="wpw_fp_options[authors_follow_buttons]"><?php esc_html_e( 'Follow Button Text', 'wpwfp' ); ?></label>
								</th>
								<td>
									<input type="text" name="wpw_fp_options[authors_follow_buttons][follow]" value="<?php echo $wpw_fp_model->wpw_fp_escape_attr( $wpw_fp_options['authors_follow_buttons']['follow'] ); ?>" placeholder="<?php esc_html_e( 'Follow', 'wpwfp' );?>"/>
									<input type="text" name="wpw_fp_options[authors_follow_buttons][following]" value="<?php echo $wpw_fp_model->wpw_fp_escape_attr( $wpw_fp_options['authors_follow_buttons']['following'] ); ?>" placeholder="<?php esc_html_e( 'Following', 'wpwfp' );?>"/>
									<input type="text" name="wpw_fp_options[authors_follow_buttons][unfollow]" value="<?php echo $wpw_fp_model->wpw_fp_escape_attr( $wpw_fp_options['authors_follow_buttons']['unfollow'] ); ?>" placeholder="<?php esc_html_e( 'UnFollow', 'wpwfp' );?>"/></br>
									<span class="description"><?php printf( esc_html__( 'Enter Follow, Following and Unfollow button text respectively. Default button text are Follow, Following and Unfollow respectively. %s - displays the name of the author.', 'wpwfp' ), '<br /><code>{author_name}</code>' ); ?></span>
								</td>
							</tr>
							
							<tr valign="top">
								<th scope="row">
									<label for="wpw_fp_options[authors_follow_message]"><?php esc_html_e( 'Followers Counter Message', 'wpwfp' ); ?></label>
								</th>
								<td>
									<input type="text" name="wpw_fp_options[authors_follow_message]" id="authors_follow_message" value="<?php echo $wpw_fp_model->wpw_fp_escape_attr( $wpw_fp_options['authors_follow_message'] ); ?>" size="76" /></br>
									<span class="description"><?php printf( esc_html__( 'Here you can give the custom message to show followers count for Authors. Leave it blank to hide the followers counter.%s - displays the followers count.', 'wpwfp' ), '<br /><code>{followers_count}</code>' ); ?></span>
								</td>
							</tr>
							<!-- Author Follow Settings End -->
							
							<!-- Page Settings Start -->
							<tr>
								<th colspan="2">
									<h3><?php esc_html_e( 'Pages Settings', 'wpwfp' ); ?></h3>
								</th>
							</tr>
							
							<tr>
								<th scope="row">
									<label for="wpw_fp_options[subscribe_manage_page]"><?php esc_html_e( 'Subscription Manage Page', 'wpwfp' );?></label>
								</th>
								<td>
									<select id="wpw_fp_options[subscribe_manage_page]" name="wpw_fp_options[subscribe_manage_page]" class="chosen-select">
										<option value=""><?php esc_html_e('--Select A Page--','wpwfp');?></option>
										<?php foreach ( $get_pages as $page ) { ?>
												<option value="<?php echo $page->ID;?>" <?php selected( $page->ID, $wpw_fp_options['subscribe_manage_page'], true ); ?>><?php echo $page->post_title;?></option>
										<?php } ?>
									</select><br />
									<span class="description"><?php esc_html_e( 'This is subscription manage page whcih shows followed posts list and followed terms list to user.', 'wpwfp' ); ?></span>
								</td>
							</tr>
							
							<tr>
								<th scope="row">
									<label for="wpw_fp_options[unsubscribe_page]"><?php esc_html_e( 'Unsubscription Manage Page', 'wpwfp' );?></label>
								</th>
								<td>
									<select id="wpw_fp_options[unsubscribe_page]" name="wpw_fp_options[unsubscribe_page]" class="chosen-select">
										<option value=""><?php esc_html_e('--Select A Page--','wpwfp');?></option>
										<?php foreach ( $get_pages as $page ) { ?>
												<option value="<?php echo $page->ID;?>" <?php selected( $page->ID, $wpw_fp_options['unsubscribe_page'], true ); ?>><?php echo $page->post_title;?></option>
										<?php } ?>
									</select><br />
									<span class="description"><?php esc_html_e( 'This is unsubscription manage Page whcih contains the form to unsubscribe the email address for the users.', 'wpwfp' ); ?></span>
								</td>
							</tr>
							<!-- Misc Setting Starts -->
							<tr>
								<th colspan="2">
									<h3><?php esc_html_e('Misc Settings', 'wpwfp'); ?></h3>
								</th>
							</tr>
							<?php 
							//Check if BuddyPress plugin active
					        if( class_exists( 'BuddyPress' ) ) {?>
							<tr valign="top">
								<th scope="row">
									<label for="wpw_fp_options[enabled_supports]"><?php esc_html_e( 'Enable Support for BuddyPress', 'wpwfp' ); ?></label>
								</th>
								<td>
									<ul>
										<li class="wpwfp-disable-meta-box">
											<input type="checkbox" id="wpwfp_enable_buddypress_support" name="wpw_fp_options[enabled_supports][]" value="buddypress" <?php if ( isset( $wpw_fp_options['enabled_supports'] ) && in_array( 'buddypress', $wpw_fp_options['enabled_supports'] ) ) { echo 'checked'; } ?>/>
											<label for="wpwfp_enable_buddypress_support"><?php echo sprintf( esc_html__( 'Check this box if you want to enable support for BuddyPress. ' ) );?></label>
										</li>
									</ul>
								</td>
							</tr>
							<?php }?>
							<tr valign="top">
								<th scope="row">
									<label for="wpw_auto_poster_options[autopost_thirdparty_plugins]"><?php esc_html_e( 'Add support to work with thirdparty plugins', 'wpwfp' ); ?></label>
								</th>
								<td>
									<input id="wpw_fp_options[autopost_thirdparty_plugins]" name="wpw_fp_options[autopost_thirdparty_plugins]" type="checkbox" value="1" <?php if ( isset( $wpw_fp_options['autopost_thirdparty_plugins'] ) ) { checked( '1', $wpw_fp_options['autopost_thirdparty_plugins'] ); } ?> />
									<span class="description"><?php esc_html_e( 'Check this box if you wanted to allow to send notifications to all followers when any page/post created from frontside from any thirdparty plugins.', 'wpwfp' ); ?></span>
								</td>
							</tr>
							<tr>
								<th scope="row">
									<label for="wpw_fp_options[custom_css]"><?php esc_html_e( 'Custom CSS', 'wpwfp' );?></label>
								</th>
								<td>
									<textarea id="wpw_fp_options[custom_css]" name="wpw_fp_options[custom_css]" style="width:100%;min-height:100px;"><?php echo !empty( $wpw_fp_options['custom_css'] ) ? $wpw_fp_options['custom_css'] : '';?></textarea><br />
									<span class="description"><?php esc_html_e( 'Here you can enter your custom css for the follow my blog post. The css will automatically added to the header, when you save it.', 'wpwfp' ); ?></span>
								</td>
							</tr>
							<!-- Misc Setting Ends -->
							<!-- Page Settings End --><?php
							
							// do action for add setting after general settings
							do_action( 'wpw_fp_after_general_setting', $wpw_fp_options );
							
							?>
							<tr>
								<td colspan="2"><?php
									echo apply_filters ( 'wpweb_fb_settings_submit_button', '<input class="button-primary wpw-fp-save-btn" type="submit" name="wpw-fp-set-submit" value="'.esc_html__( 'Save Changes','wpwfp' ).'" />' );?>
								</td>
							</tr>
						</tbody>
					</table>
				</div><!-- .inside -->
			</div><!-- #general -->
		</div><!-- .meta-box-sortables ui-sortable -->
	</div><!-- .metabox-holder -->
</div><!-- #wpw-fp-general -->