<?php


/**
 * Enqueue the block's assets for the editor.
 *
 */
function follow_my_blog_posts_editor_assets()
{	
	global $post;
	// Enqueue block editor scripts
	wp_register_script(
		'follow-my-blog-posts-block-js',
		WPW_FP_BLOCK_URL . 'follow-my-blog-posts/blocks.js',
		array('wp-blocks', 'wp-components', 'wp-element', 'wp-i18n', 'wp-editor'),
		filemtime(WPW_FP_BLOCK_DIR . 'follow-my-blog-posts/blocks.js')
	);

	wp_register_style( 'wpw-fp-public-style', WPW_FP_URL . 'includes/css/wpw-fp-public.css', array(), WPW_FP_VERSION );
	wp_enqueue_style( 'wpw-fp-public-style' );

	/* Term Follow Button */
	$post_types = get_post_types( array( 'public' => true ), 'objects' );
	$taxonomy_type_array = [];
	foreach ( $post_types as $post_key => $post_type ) { 
											
		if( $post_key != 'attachment' ) { // Check not media type
			
			$all_taxonomy = get_object_taxonomies( $post_key );
			if( !empty( $all_taxonomy ) ) { // Check taxonomy is not empty
				$taxonomy_type_array[$post_type->labels->name]["optgroup"][] = $post_type->labels->name;
				foreach ( $all_taxonomy as $taxonomy_slug ) {
					if( $taxonomy_slug != 'post_format' ) {
						$tax = get_taxonomy( $taxonomy_slug );
						$taxonomy_type_array[$post_type->labels->name]["option"][] = array(
							'value' => $taxonomy_slug,
							'data_posttype' => $post_key,
							'label' => $tax->label,
						);
					}
				}
			}
		}
	}
	
	/* Author Follow Button */
	$found_authors = array();
	$authors_query = new WP_User_Query(array(
		'fields' => ["ID", "display_name", "user_email"],
		'orderby' => 'display_name',
		'number' => '-1'

	));

	$authors = $authors_query->get_results();

	if ($authors) {
		foreach ($authors as $author) {
			$found_authors[$author->ID] = $author->display_name . ' (#' . $author->ID . ' - ' . sanitize_email($author->user_email) . ')';
		}
	}
	$fmbpbjs = array(
		'wpw_fp_author_nm' => $found_authors,
		'wpw_fp_term_taxonomy' => $taxonomy_type_array,
		'wpw_fp_post_id' => $post->ID,
	);
	wp_localize_script('follow-my-blog-posts-block-js', 'fmbpbjs', $fmbpbjs);

	// Enqueue block editor styles
	wp_enqueue_style(
		'follow-my-blog-posts-block-css',
		WPW_FP_BLOCK_URL . 'follow-my-blog-posts/follow-my-blog-post-block.css',
		filemtime(WPW_FP_BLOCK_DIR . 'follow-my-blog-posts/follow-my-blog-post-block.css')
	);

}
add_action('enqueue_block_editor_assets', 'follow_my_blog_posts_editor_assets');


/**
 * Handle Woocommerce - Social Login Block Registering
 *
 */
function follow_my_blog_posts_register_block()
{
	
	if (function_exists('register_block_type')) {

		$args = array(

			'editor_script' => 'follow-my-blog-posts-block-js',
			'attributes' => array(
				'wpw_fp_shortcodes' => array(
					'type' => 'string',
					'default' => '',
					'items' => array(
						'type' => 'string',
					)
				),
				'wpw_fp_post_id' => array(
					'type' => 'text',
					'default' => esc_html__('', 'wpwfp'),
				),
				'wpw_fp_post_id_hidden' => array(
					'type' => 'text',
					'default' => esc_html__('', 'wpwfp'),
				),
				'wpw_fp_disable_followers_count' => array(
					'type' => 'boolean',
					'default' => false,
				),
				'wpw_fp_followers_count_msg' => array(
					'type' => 'text',
					'default' => esc_html__('', 'wpwfp'),
				),
				'wpw_fp_follow_text' => array(
					'type' => 'text',
					'default' => '',
				),
				'wpw_fp_following_text' => array(
					'type' => 'text',
					'default' => '',
				),
				'wpw_fp_unfollow_text' => array(
					'type' => 'text',
					'default' => esc_html__('', 'wpwfp'),
				),

				/* Author Follow button */
				'wpw_fp_author_nm' => array(
					'type' => 'string',
					'default' => '',
					'items' => array(
						'type' => 'string',
					)
				),
				'wpw_fp_author_disable_followers_count' => array(
					'type' => 'boolean',
					'default' => false,
				),
				'wpw_fp_author_followers_count_msg' => array(
					'type' => 'text',
					'default' => esc_html__('', 'wpwfp'),
				),
				'wpw_fp_author_follow_text' => array(
					'type' => 'text',
					'default' => esc_html__('', 'wpwfp'),
				),
				'wpw_fp_author_following_text' => array(
					'type' => 'text',
					'default' => esc_html__('', 'wpwfp'),
				),
				'wpw_fp_author_unfollow_text' => array(
					'type' => 'text',
					'default' => esc_html__('', 'wpwfp'),
				),

				/* Term Follow button */
				'wpw_fp_term_taxonomy_chosen' => array(
					'type' => 'string',
					'default' => '',
					'items' => array(
						'type' => 'string',
					)
				),
				'wpw_fp_term_term_id_chosen' => array(
					'type' => 'text',
					'default' => esc_html__('', 'wpwfp'),
				),
				'wpw_fp_term_disable_followers_count' => array(
					'type' => 'boolean',
					'default' => false,
				),
				'wpw_fp_term_followers_count_msg' => array(
					'type' => 'text',
					'default' => esc_html__('', 'wpwfp'),
				),
				'wpw_fp_term_follow_text' => array(
					'type' => 'text',
					'default' => esc_html__('', 'wpwfp'),
				),
				'wpw_fp_term_following_text' => array(
					'type' => 'text',
					'default' => esc_html__('', 'wpwfp'),
				),
				'wpw_fp_term_unfollow_text' => array(
					'type' => 'text',
					'default' => esc_html__('', 'wpwfp'),
				),

				/* Posts button */
				'wpw_fp_only_following_post' => array(
					'type' => 'boolean',
					'default' => false,
				),

				/* Terms button */
				'wpw_fp_only_following_term' => array(
					'type' => 'boolean',
					'default' => false,
				),

				/* Authors button */
				'wpw_fp_only_following_author' => array(
					'type' => 'boolean',
					'default' => false,
				),

			),
			'render_callback' => 'follow_my_blog_posts_block_render',
		);
    
		// register woocommerce social login custom block
		register_block_type('wpweb/follow-my-blog-posts-block', $args);
	}

}
add_action('init', 'follow_my_blog_posts_register_block');


/**
 * Handle  Block Rendering
 *
 */
function follow_my_blog_posts_block_render($attributes)
{	
	global $wpw_fp_options;

	if ($attributes["wpw_fp_shortcodes"] == "") {
		$args = array();
		$content = esc_html__("Please select any shortcode", 'wpwfp');
		return apply_filters('wpw_fp_follow_shortcode_content', $content, $args);
	}
	if ($attributes["wpw_fp_shortcodes"] == "wpw_follow_me") {
		$wpw_fp_shortcodes = new Wpw_Fp_Shortcodes;
		
		if(empty($attributes["wpw_fp_post_id"])){
			$attributes["wpw_fp_post_id"] = $attributes["wpw_fp_post_id_hidden"];
		}
		
		$shortcode_atts = array(
			'id' => $attributes["wpw_fp_post_id"],
			'disablecount' => $attributes["wpw_fp_disable_followers_count"],
			'followerscountmsg' => $attributes["wpw_fp_followers_count_msg"],
			'followtext' => $attributes["wpw_fp_follow_text"],
			'followingtext' => $attributes["wpw_fp_following_text"],
			'unfollowtext' => $attributes["wpw_fp_unfollow_text"],
			'disable_reload'	=>	false,
		);

		return $wpw_fp_shortcodes->wpw_fp_follow_shortcode($shortcode_atts, '');
	}
	/* Author Follow button */
	if ($attributes["wpw_fp_shortcodes"] == "wpw_follow_author_me") {
		global $post;
		if (!empty($attributes["wpw_fp_author_nm"])) {
			$post_author = $attributes["wpw_fp_author_nm"];
		} else {
			$post_author = isset($post->post_author) ? $post->post_author : '';
			//Check when post author empty, consider as author page
			if (empty($post_author)) {
				$post_author = get_current_user_id();
			}
		}

		$wpw_fp_shortcodes = new Wpw_Fp_Shortcodes;
		
		$shortcode_atts = array(
			'author_id' => $post_author,
			'disablecount' => $attributes["wpw_fp_author_disable_followers_count"],
			'followerscountmsg' => $attributes["wpw_fp_author_followers_count_msg"],
			'followtext' => $attributes["wpw_fp_author_follow_text"],
			'followingtext' => $attributes["wpw_fp_author_following_text"],
			'unfollowtext' => $attributes["wpw_fp_author_unfollow_text"],
		);

		return $wpw_fp_shortcodes->wpw_fp_follow_author_shortcode($shortcode_atts, '');
	}

	/* Term Follow button */
	if ($attributes["wpw_fp_shortcodes"] == "wpw_follow_term_me") {

		if(empty($attributes["wpw_fp_term_taxonomy_chosen"])){
			$args = array();
			$content = esc_html__( "Please select taxonomy.", 'wpwfp' );
			return apply_filters( 'wpw_fp_follow_term_shortcode_content', $content, $args );
		}

		if(empty($attributes["wpw_fp_term_term_id_chosen"])){
			$args = array();
			$content = esc_html__( "Please enter term id.", 'wpwfp' );
			return apply_filters( 'wpw_fp_follow_term_shortcode_content', $content, $args );
		}

		$attributes["wpw_fp_term_taxonomy_chosen"] = explode("$$", $attributes["wpw_fp_term_taxonomy_chosen"]);
		$wpw_fp_shortcodes = new Wpw_Fp_Shortcodes;
		$shortcode_atts = array(
			'posttype'			=>	$attributes["wpw_fp_term_taxonomy_chosen"][0],
	    	'taxonomy'			=>	$attributes["wpw_fp_term_taxonomy_chosen"][1],
	    	'termid'			=>	$attributes["wpw_fp_term_term_id_chosen"],
	    	'disablecount'		=> 	$attributes["wpw_fp_term_disable_followers_count"],
	    	'followerscountmsg'	=> 	$attributes["wpw_fp_term_followers_count_msg"],
	    	'followtext'		=>	$attributes["wpw_fp_term_follow_text"],
	    	'followingtext'		=>	$attributes["wpw_fp_term_following_text"],
	    	'unfollowtext'		=>	$attributes["wpw_fp_term_unfollow_text"],
		);
		
		return $wpw_fp_shortcodes->wpw_fp_follow_term_shortcode($shortcode_atts, '');
	}

	/* Posts button */
	if ($attributes["wpw_fp_shortcodes"] == "wpw_follow_post_list") {
		$wpw_fp_shortcodes = new Wpw_Fp_Shortcodes;
		$shortcode_atts = array(
			'only_following' => ($attributes["wpw_fp_only_following_post"]) ? 1 : ""
		);

		return $wpw_fp_shortcodes->wpw_fp_follow_post_list_shortcode($shortcode_atts, '');
	}

	/* Terms button */
	if ($attributes["wpw_fp_shortcodes"] == "wpw_follow_term_list") {
		$wpw_fp_shortcodes = new Wpw_Fp_Shortcodes;
		$shortcode_atts = array(
			'only_following' => ($attributes["wpw_fp_only_following_term"]) ? 1 : ""
		);

		return $wpw_fp_shortcodes->wpw_fp_follow_term_list_shortcode($shortcode_atts, '');
	}

	/* Authors button */
	if ($attributes["wpw_fp_shortcodes"] == "wpw_follow_author_list") {
		$wpw_fp_shortcodes = new Wpw_Fp_Shortcodes;
		$shortcode_atts = array(
			'only_following' => ($attributes["wpw_fp_only_following_author"]) ? 1 : ""
		);

		return $wpw_fp_shortcodes->wpw_fp_follow_author_list_shortcode($shortcode_atts, '');
	}

}