<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

/**
 * Scripts Class
 *
 * Handles adding scripts functionality to the admin pages
 * as well as the front pages.
 *
 * @package Follow My Blog Post
 * @since 1.0.0
 */
class Wpw_Fp_Scripts {
	
	public function __construct() {
		
	}
	
	/**
	 * Enqueue Styles for admin
	 * 
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	public function wpw_fp_admin_print_styles( $hook_suffix ) {

		//Get main page 
		$main_page_slug = sanitize_title( esc_html__( 'Follow My Blog Post', 'wpwfp' ) );

		$pages_hook_suffix = array( 'post.php', 'post-new.php', $main_page_slug .'_page_wpw-fp-add-follower', $main_page_slug .'_page_wpw-fp-send-email', $main_page_slug .'_page_wpw-fp-settings', 'toplevel_page_wpw-fp-post', $main_page_slug .'_page_wpw-fp-term', $main_page_slug .'_page_wpw-fp-author', 'toplevel_page_wpw-fp-author', $main_page_slug .'_page_wpw-fp-extensions' );

		//Check pages when you needed
		if( in_array( $hook_suffix, $pages_hook_suffix ) ) {
			
			wp_register_style( 'chosen-style', WPW_FP_URL . 'includes/css/chosen/chosen.css', array(), WPW_FP_VERSION );
			wp_enqueue_style( 'chosen-style' );
			
			wp_register_style( 'chosen-custom-style', WPW_FP_URL . 'includes/css/chosen/chosen-custom.css', array(), WPW_FP_VERSION );
			wp_enqueue_style( 'chosen-custom-style' );
			
			wp_register_style( 'wpw-fp-admin-styles', WPW_FP_URL . 'includes/css/wpw-fp-admin.css', array(), WPW_FP_VERSION );
			wp_enqueue_style( 'wpw-fp-admin-styles' );
		
			wp_register_style( 'wpw-fp-popup-styles', WPW_FP_URL . 'includes/css/wpw-fp-prettyphoto.css', array(), WPW_FP_VERSION );
			wp_enqueue_style( 'wpw-fp-popup-styles' );
		}
		
		wp_register_style( 'wpw-fp-admin-general-styles', WPW_FP_URL . 'includes/css/wpw-fp-general-admin.css', array(), WPW_FP_VERSION );
		wp_enqueue_style( 'wpw-fp-admin-general-styles' );
	}
	
	/**
	 * Enqueue Scripts for backend
	 * 
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	public function wpw_fp_admin_enqueue_scripts( $hook_suffix ) {
		
		global $wp_version;

		//Get main page 
		$main_page_slug = sanitize_title( esc_html__( 'Follow My Blog Post', 'wpwfp' ) );

		$pages_hook_suffix = array( 'post.php', 'post-new.php', $main_page_slug .'_page_wpw-fp-add-follower', $main_page_slug .'_page_wpw-fp-send-email', $main_page_slug .'_page_wpw-fp-settings', 'toplevel_page_wpw-fp-post', $main_page_slug .'_page_wpw-fp-term', $main_page_slug .'_page_wpw-fp-author' );

		//Check pages when you needed
		if( in_array( $hook_suffix, $pages_hook_suffix ) ) {

			wp_register_script( 'chosen', WPW_FP_URL . 'includes/js/chosen/chosen.jquery.js', array( 'jquery' ), WPW_FP_VERSION, true );
			wp_enqueue_script( 'chosen' );
			
			wp_register_script( 'ajax-chosen', WPW_FP_URL . 'includes/js/chosen/ajax-chosen.jquery.js', array( 'jquery' ), WPW_FP_VERSION, true );
			wp_enqueue_script( 'ajax-chosen' );
			
			wp_register_script( 'wpw-fp-admin-scripts', WPW_FP_URL . 'includes/js/wpw-fp-admin.js', array('jquery','jquery-ui-datepicker', 'jquery-ui-sortable' ) , WPW_FP_VERSION, true );
			wp_enqueue_script( 'wpw-fp-admin-scripts' );
			
			//localize script
			$newui = $wp_version >= '3.5' ? '1' : '0'; //check wp version for showing media uploader
			
			wp_localize_script( 'wpw-fp-admin-scripts', 'WpwFpSettings', array( 
																					'new_media_ui' 		=> $newui,
																					'resetmsg'			=> esc_html__( 'Click OK to reset all options. All settings will be lost!', 'wpwfp' ),
																					'testemailsuccess'	=> esc_html__( 'Test email has been sent successfully.', 'wpwfp' ),
																					'testemailerror'	=> esc_html__( 'Test email could not sent.', 'wpwfp' )
																				) );
			if (function_exists('wp_enqueue_media')){
				wp_enqueue_media();
			}
			
		
			wp_register_script( 'wpw-fp-popup-scripts', WPW_FP_URL . 'includes/js/wpw-fp-prettyphoto.js', array('jquery'), WPW_FP_VERSION );
			wp_enqueue_script( 'wpw-fp-popup-scripts' );
			
			// loads the required scripts for the meta boxes
			wp_enqueue_script( 'common' );
			wp_enqueue_script( 'wp-lists' );
			wp_enqueue_script( 'postbox' );
				
		}
	}

	/**
	 * Enqueue Scripts for widget
	 * 
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	public function wpw_fp_widget_scripts( $hook_suffix ) {

		if( in_array( $hook_suffix, array( 'widgets.php' ) ) ) {

			// Register & Enqueue widget script
			wp_register_script( 'wpw-fp-widget-script',  WPW_FP_URL .'includes/js/wpw-fp-widget.js', array(), WPW_FP_VERSION , true );
			wp_enqueue_script( 'wpw-fp-widget-script' );		
		}
	}

	/**
	 * Enqueue Styles for public
	 * 
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	public function wpw_fp_public_print_styles() {
		
		global $wpw_fp_options;
		
		wp_register_style( 'wpw-fp-public-style', WPW_FP_URL . 'includes/css/wpw-fp-public.css', array(), WPW_FP_VERSION );
		wp_enqueue_style( 'wpw-fp-public-style' );

		// added since 1.8.6
		wp_register_style( 'chosen-style', WPW_FP_URL . 'includes/css/chosen/chosen.css', array(), WPW_FP_VERSION );
		wp_register_style( 'chosen-custom-style', WPW_FP_URL . 'includes/css/chosen/chosen-custom.css', array(), WPW_FP_VERSION );
		wp_register_style( 'wpw-fp-admin-styles', WPW_FP_URL . 'includes/css/wpw-fp-admin.css', array(), WPW_FP_VERSION );
	}
	
	/**
	 * Loading Additional Java Script
	 *
	 * Loads the JavaScript required for toggling the meta boxes on the theme settings page.
	 *
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	public function wpw_fp_settings_page_load_scripts( $hook_suffix ) { 

		//Get main menu slug
		$main_page_slug = sanitize_title( esc_html__( 'Follow My Blog Post', 'wpwfp' ) );

		wp_register_script( 'wpw-fp-settings-page-scripts', WPW_FP_URL . 'includes/js/wpw-fp-settings-page.js', array('jquery' ) , WPW_FP_VERSION, true );
		wp_enqueue_script( 'wpw-fp-settings-page-scripts' );
		
		wp_localize_script( 'wpw-fp-settings-page-scripts', 'WpwFpSettingsPage', 
			array( 
				'main_page_slug' => $main_page_slug
			) 
		);

	}
	
	/**
	 * Loading Additional Java Script
	 *
	 * Loads the JavaScript required for toggling the meta boxes on the theme settings page.
	 *
	 * @package Follow My Blog Post
	 * @since 1.4.4
	 */
	public function wpw_fp_send_email_page_load_scripts( $hook_suffix ) { 

		//Get main menu slug
		$main_page_slug = sanitize_title( esc_html__( 'Follow My Blog Post', 'wpwfp' ) );
		wp_register_script( 'wpw-fp-sent-email-page-scripts', WPW_FP_URL . 'includes/js/wpw-fp-sent-email-page.js', array('jquery' ) , WPW_FP_VERSION, true );
		wp_enqueue_script( 'wpw-fp-sent-email-page-scripts' );
		
		wp_localize_script( 'wpw-fp-sent-email-page-scripts', 'WpwFpSentEmailPage', 
			array( 
				'main_page_slug' => $main_page_slug
			) 
		);
	}
	
	/**
	 * Enqueue Scripts for frontside
	 * 
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	public function wpw_fp_front_scripts() {

		global $wpw_fp_options, $post;
		
		wp_register_script( 'wpw-fp-public-scripts', WPW_FP_URL . 'includes/js/wpw-fp-public.js', array('jquery' ) , WPW_FP_VERSION, true );
		wp_enqueue_script( 'wpw-fp-public-scripts' );
		
		$loggin_flag = is_user_logged_in() ? '1' : '0';
		wp_localize_script( 'wpw-fp-public-scripts', 'WpwFpPublic', array( 
																				'ajaxurl'		=> admin_url('admin-ajax.php'),
																				'emailempty' 	=> esc_html__( 'Please enter email.', 'wpwfp' ),
																				'emailinvalid'	=> esc_html__( 'Please enter valid email.', 'wpwfp' ),
																				'loginflag'		=> $loggin_flag,
																				'checkemail'	=> esc_html__( 'Please check your email inbox to confirm subscription.', 'wpwfp' ),
																				'processing'	=> esc_html__( 'Processing', 'wpwfp')
																			) );

		// added since 1.8.6
		wp_register_script( 'chosen', WPW_FP_URL . 'includes/js/chosen/chosen.jquery.js', array( 'jquery' ), WPW_FP_VERSION, true );
			
		wp_register_script( 'ajax-chosen', WPW_FP_URL . 'includes/js/chosen/ajax-chosen.jquery.js', array( 'jquery' ), WPW_FP_VERSION, true );
		wp_register_script( 'wpw-fp-admin-scripts', WPW_FP_URL . 'includes/js/wpw-fp-admin.js', array('jquery','jquery-ui-datepicker', 'jquery-ui-sortable' ) , WPW_FP_VERSION, true );
	}
	
	/**
	 * Display button in post / page container
	 *
	 * Handles to display button in post / page container
	 * 
	 * @package Follow My Blog Post
	 * @since 1.1.0
	 */
	public function wpw_fp_shortcode_display_button( $buttons ) {
	 
		array_push( $buttons, "|", "wpw_fp_follow_post" );
		return $buttons;
	}
	
	/**
	 * Include js for add button in post / page container
	 *
	 * Handles to include js for add button in post / page container
	 * 
	 * @package Follow My Blog Post
	 * @since 1.1.0
	 */
	public function wpw_fp_shortcode_button($plugin_array) {
	 
		$plugin_array['wpw_fp_follow_post'] = WPW_FP_URL . 'includes/js/wpw-fp-shortcodes.js?ver='.WPW_FP_VERSION;
		return $plugin_array;
	}
	
	/**
	 * Display button in post / page container
	 * 
	 * Handles to display button in post / page container
	 * 
	 * @package Follow My Blog Post
	 * @since 1.1.0
	 */
	public function wpw_fp_add_shortcode_button() {
		
		if( current_user_can( 'manage_options' ) || current_user_can( 'edit_posts' ) ) {
			add_filter( 'mce_external_plugins', array( $this, 'wpw_fp_shortcode_button' ) );
   			add_filter( 'mce_buttons', array( $this, 'wpw_fp_shortcode_display_button' ) );
		}
		
	}

	/**
	 * style on head of page
	 * 
	 * Handles style code display when wp head initialize
	 * 
	 * @package Follow My Blog Post
 	 * @since 1.8.5
	 */
	public function wpw_fp_custom_style() {
		
		//Get custom css code
		global $wpw_fp_options;
		
		if( !empty( $wpw_fp_options['custom_css'] ) )	{//if custom css code not available
			echo '<style type="text/css">' . $wpw_fp_options['custom_css'] . '</style>';
		}
	}

	/**
	 * Adding Hooks
	 *
	 * Adding proper hoocks for the scripts.
	 *
	 * @package Follow My Blog Post
	 * @since 1.0.0
	 */
	public function add_hooks() {
		
		//add styles for admin
		add_action( 'admin_enqueue_scripts', array( $this, 'wpw_fp_admin_print_styles' ) );
		
		//add scripts for admin 
		add_action( 'admin_enqueue_scripts', array( $this, 'wpw_fp_admin_enqueue_scripts') );

		//script for follow button widget
		add_action( 'admin_enqueue_scripts', array( $this, 'wpw_fp_widget_scripts' ) );

		//add styles for public
		add_action( 'wp_enqueue_scripts', array( $this, 'wpw_fp_public_print_styles' ) );
		
		//script for front side
		add_action( 'wp_enqueue_scripts', array( $this, 'wpw_fp_front_scripts' ) );
		
		// add filters for add add button in post / page container
		add_action( 'admin_init', array( $this, 'wpw_fp_add_shortcode_button' ) );

		//style code on wp head
		add_action( 'wp_head', array( $this, 'wpw_fp_custom_style' ) );
	}
}