<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

add_action( 'widgets_init', 'wpw_fp_follow_author_widget' );

/**
 * Register the Author Follow Button Widget
 *
 * @package Follow My Blog Post
 * @since 1.8.5
 */
function wpw_fp_follow_author_widget() {
	register_widget( 'Wpw_Fp_Follow_Author_Widget' );
}

/**
 * Widget Class
 *
 * Handles generic functionailties
 *
 * @package Follow My Blog Post
 * @since 1.8.5
 */
class Wpw_Fp_Follow_Author_Widget extends WP_Widget {
	
	public $model;
	
	public function __construct() {

		global $wpw_fp_model;

		$widget_ops = array('classname' => 'widget_text', 'description' => esc_html__( 'Display the follow author button.', 'wpwfp' ) );		
		WP_Widget::__construct( 'wpw-fp-follow-author-widget', esc_html__('Follow Author Button', 'wpwfp'), $widget_ops);
		$this->model = $wpw_fp_model;
	}

	public function form($instance) {
		
		// outputs the options form on admin
		$instance = wp_parse_args( (array) $instance, array( 'title' => '', 'authorid' => '', 'disablefollowerscount' =>'', 'followerscountmsg' => '', 'followtext' => '', 'followingtext' => '', 'unfollowtext' => '' ));
		$title = $this->model->wpw_fp_escape_attr($instance['title']);
		$author_id = $this->model->wpw_fp_escape_attr($instance['authorid']);
		$disable_followers_count = $instance['disablefollowerscount'];
		$followers_count_msg = $this->model->wpw_fp_escape_attr($instance['followerscountmsg']);
		$follow_text = $this->model->wpw_fp_escape_attr($instance['followtext']);
		$following_text = $this->model->wpw_fp_escape_attr($instance['followingtext']);
		$unfollow_text = $this->model->wpw_fp_escape_attr($instance['unfollowtext']);
		?>
		
		<!-- Title: Text Box -->
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>"><?php esc_html_e('Title:', 'wpwfp'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>" />
		</p>
		
		<!-- Author Name: Text Box -->
		<p>
			<label for="<?php echo $this->get_field_id('authorid'); ?>"><?php esc_html_e('Select Author:', 'wpwfp'); ?></label>
			<select id="<?php echo $this->get_field_id('authorid'); ?>" name="<?php echo $this->get_field_name('authorid'); ?>" placeholder="<?php esc_html_e( 'Choose an Author...','wpwfp');?>" class="widefat wpw_fp_author_name chosen-select">
		    	<option value=""><?php esc_html_e( 'Choose an Author...','wpwfp');?></option>
		    	<?php 

		    		$authors_query = new WP_User_Query( array(
						'fields'			=> 'all',
						'orderby'			=> 'display_name'
					) );
				
					$authors = $authors_query->get_results();
				
					if ( $authors ) {
						foreach ( $authors as $author ) {
							?>
							<option value="<?php echo $author->ID; ?>" <?php selected( $author_id, $author->ID ); ?> ><?php echo $author->display_name; ?></option>
							<?php
						}
					}
					
				?>
		    </select>
		</p>

		<!-- Disable Followers Count Checkbox -->
		<p>
			<label for="<?php echo $this->get_field_id( 'disablefollowerscount' ); ?>"><?php esc_html_e('Disable Followers Count', 'wpwfp'); ?></label>
			<input class="widefat wpw_fp_disable_followers_count" type="checkbox" value="1" <?php checked( $disable_followers_count, 1 ); ?> id="<?php echo $this->get_field_id( 'disablefollowerscount' ); ?>" name="<?php echo $this->get_field_name( 'disablefollowerscount' ); ?>" /><br />
			<span class="description wpwfp-desc"><?php esc_html_e( 'Check this box if you want to hide followers count.', 'wpwfp' );?></span>
		</p>
		
		<!-- Followers Counter Message Text Box -->
		<p class="wpw-fp-disable-count-msg-wrap" style="<?php echo !empty( $disable_followers_count ) ? 'display:none;' : '';?>">
			<label for="<?php echo $this->get_field_id('followerscountmsg'); ?>"><?php esc_html_e('Followers Counter Message:', 'wpwfp'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('followerscountmsg'); ?>" name="<?php echo $this->get_field_name('followerscountmsg'); ?>" type="text" value="<?php echo esc_attr($followers_count_msg); ?>" /><br />
			<span class="description wpwfp-desc"><?php printf( esc_html__( 'Here you can give the custom message to show followers count. Leave it empty to use the default one from the settings page.%s - displays the followers count.', 'wpwfp' ), '<br><code>{followers_count}</code>');?></span>
		</p>
		
		<!-- Follow Button Text Text Box -->
		<p>
			<label ><?php esc_html_e('Follow Button Text:', 'wpwfp'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('followtext'); ?>" name="<?php echo $this->get_field_name('followtext'); ?>" type="text" value="<?php echo esc_attr($follow_text); ?>" placeholder="<?php esc_html_e( 'Follow', 'wpwfp' );?>"/>
			<input class="widefat" id="<?php echo $this->get_field_id('followingtext'); ?>" name="<?php echo $this->get_field_name('followingtext'); ?>" type="text" value="<?php echo esc_attr($following_text); ?>" placeholder="<?php esc_html_e( 'Following', 'wpwfp' );?>"/>
			<input class="widefat" id="<?php echo $this->get_field_id('unfollowtext'); ?>" name="<?php echo $this->get_field_name('unfollowtext'); ?>" type="text" value="<?php echo esc_attr($unfollow_text); ?>" placeholder="<?php esc_html_e( 'UnFollow', 'wpwfp' );?>"/><br />
			<span class="description wpwfp-desc"><?php printf( esc_html__( 'Enter Follow, Following and Unfollow button text respectively. Leave it empty to use the default one from the settings page.%s - displays the name of the author.', 'wpwfp' ), '<br><code>{author_name}</code>');?></span>
		</p>

		<?php
	}

	public function update( $new_instance, $old_instance ) {

		// processes widget options to be saved
		$instance = $old_instance;
		$instance['title'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['title']);
		$instance['authorid'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['authorid']);
		$instance['disablefollowerscount'] = $new_instance['disablefollowerscount'];
		$instance['followerscountmsg'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['followerscountmsg']);
		$instance['followtext'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['followtext']);
		$instance['followingtext'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['followingtext']);
		$instance['unfollowtext'] = $this->model->wpw_fp_escape_slashes_deep($new_instance['unfollowtext']);

		return $new_instance;
	}

	public function widget( $args, $instance ) {

		global $wpw_fp_options;

		// outputs the content of the widget
		extract($args);
		$title = empty( $instance['title'] ) ? '' : $instance['title'];
		$author_id = empty( $instance['authorid'] ) ? '' : $instance['authorid'];

		//Check if disable follower count
		$followers_count_msg = '';
		if( empty( $instance['disablefollowerscount'] ) ) {
			$followers_count_msg = ! empty( $instance['followerscountmsg'] ) ? $instance['followerscountmsg'] : $wpw_fp_options['authors_follow_message'];
		}

		$follow_text = empty( $instance['followtext'] ) ? '' : $instance['followtext'];
		$following_text = empty( $instance['followingtext'] ) ? '' : $instance['followingtext'];
		$unfollow_text = empty( $instance['unfollowtext'] ) ? '' : $instance['unfollowtext'];
		
		$args = array(
					'author_id'			=> $author_id,
					'follow_message'	=> $followers_count_msg,
					'follow_buttons'	=> array(
												'follow' 	=> $follow_text,
												'following' => $following_text,
												'unfollow' 	=> $unfollow_text,
											),
					'shortcode'			=> true,
					'disable_reload'	=> true
				);

		echo $before_widget;

		if( !empty( $title ) ) echo $before_title . $title . $after_title;
		
		do_action( 'wpw_fp_follow_author', $args );	

		echo $after_widget;
	}
}
?>