<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

/**
 * Post Type Functions
 *
 * Handles all custom post types
 * 
 * @package Follow My Blog Post
 * @since 1.0.0 
 */

/**
 * Setup Follow Post PostTypes
 *
 * Registers the follow post posttypes
 * 
 * @package Follow My Blog Post
 * @since 1.0.0 
 */

function wpw_fp_register_post_types() {
	
	//follow post - post type
	$follow_post_labels = array(
						    'name'				=> esc_html__('Follow Post','wpwfp'),
						    'singular_name' 	=> esc_html__('Follow Post','wpwfp'),
						    'add_new' 			=> esc_html__('Add New','wpwfp'),
						    'add_new_item' 		=> esc_html__('Add New Follow Post','wpwfp'),
						    'edit_item' 		=> esc_html__('Edit Follow Post','wpwfp'),
						    'new_item' 			=> esc_html__('New Follow Post','wpwfp'),
						    'all_items' 		=> esc_html__('All Follow Posts','wpwfp'),
						    'view_item' 		=> esc_html__('View Follow Post','wpwfp'),
						    'search_items' 		=> esc_html__('Search Follow Post','wpwfp'),
						    'not_found' 		=> esc_html__('No follow posts found','wpwfp'),
						    'not_found_in_trash'=> esc_html__('No follow posts found in Trash','wpwfp'),
						    'parent_item_colon' => '',
						    'menu_name' 		=> esc_html__('Follow Posts','wpwfp'),
						);
	$follow_post_args = array(
						    'labels' 				=> $follow_post_labels,
						    'public' 				=> false,
						    'query_var' 			=> false,
						    'rewrite' 				=> false,
						    'capability_type' 		=> WPW_FP_POST_TYPE,
						    'hierarchical' 			=> false,
						    'supports' 				=> array( 'title' ),
						);
	
	//register follow posts post type
	register_post_type( WPW_FP_POST_TYPE, $follow_post_args );
	
	//follow post logs - post type
	$follow_post_logs_labels = array(
						    'name'				=> esc_html__('Follow Post Logs','wpwfp'),
						    'singular_name' 	=> esc_html__('Follow Post Log','wpwfp'),
						    'add_new' 			=> esc_html__('Add New','wpwfp'),
						    'add_new_item' 		=> esc_html__('Add New Follow Post Log','wpwfp'),
						    'edit_item' 		=> esc_html__('Edit Follow Post Log','wpwfp'),
						    'new_item' 			=> esc_html__('New Follow Post Log','wpwfp'),
						    'all_items' 		=> esc_html__('All Follow Post Logs','wpwfp'),
						    'view_item' 		=> esc_html__('View Follow Post Log','wpwfp'),
						    'search_items' 		=> esc_html__('Search Follow Post Log','wpwfp'),
						    'not_found' 		=> esc_html__('No follow post logs found','wpwfp'),
						    'not_found_in_trash'=> esc_html__('No follow post logs found in Trash','wpwfp'),
						    'parent_item_colon' => '',
						    'menu_name' 		=> esc_html__('Follow Post Logs','wpwfp'),
						);
	$follow_post_logs_args = array(
						    'labels' 				=> $follow_post_logs_labels,
						    'public' 				=> false,
						    'query_var' 			=> false,
						    'rewrite' 				=> false,
						    'capability_type' 		=> WPW_FP_LOGS_POST_TYPE,
						    'hierarchical' 			=> false,
						    'supports' 				=> array( 'title' )
					 	);
	
	//register follow posts logs post type
	register_post_type( WPW_FP_LOGS_POST_TYPE, $follow_post_logs_args );
	
	//follow term - post type
	$follow_term_labels = array(
						    'name'				=> esc_html__('Follow Term','wpwfp'),
						    'singular_name' 	=> esc_html__('Follow Term','wpwfp'),
						    'add_new' 			=> esc_html__('Add New','wpwfp'),
						    'add_new_item' 		=> esc_html__('Add New Follow Term','wpwfp'),
						    'edit_item' 		=> esc_html__('Edit Follow Term','wpwfp'),
						    'new_item' 			=> esc_html__('New Follow Term','wpwfp'),
						    'all_items' 		=> esc_html__('All Follow Terms','wpwfp'),
						    'view_item' 		=> esc_html__('View Follow Term','wpwfp'),
						    'search_items' 		=> esc_html__('Search Follow Term','wpwfp'),
						    'not_found' 		=> esc_html__('No follow terms found','wpwfp'),
						    'not_found_in_trash'=> esc_html__('No follow terms found in Trash','wpwfp'),
						    'parent_item_colon' => '',
						    'menu_name' 		=> esc_html__('Follow Terms','wpwfp'),
						);
	$follow_term_args = array(
						    'labels' 				=> $follow_term_labels,
						    'public' 				=> false,
						    'query_var' 			=> false,
						    'rewrite' 				=> false,
						    'capability_type' 		=> WPW_FP_TERM_POST_TYPE,
						    'hierarchical' 			=> false,
						    'supports' 				=> array( 'title' )
						);
	
	//register follow term post type
	register_post_type( WPW_FP_TERM_POST_TYPE, $follow_term_args );
	
	//follow term logs - post type
	$follow_term_logs_labels = array(
						    'name'				=> esc_html__('Follow Term Logs','wpwfp'),
						    'singular_name' 	=> esc_html__('Follow Term Log','wpwfp'),
						    'add_new' 			=> esc_html__('Add New','wpwfp'),
						    'add_new_item' 		=> esc_html__('Add New Follow Term Log','wpwfp'),
						    'edit_item' 		=> esc_html__('Edit Follow Term Log','wpwfp'),
						    'new_item' 			=> esc_html__('New Follow Term Log','wpwfp'),
						    'all_items' 		=> esc_html__('All Follow Term Logs','wpwfp'),
						    'view_item' 		=> esc_html__('View Follow Term Log','wpwfp'),
						    'search_items' 		=> esc_html__('Search Follow Term Log','wpwfp'),
						    'not_found' 		=> esc_html__('No follow term logs found','wpwfp'),
						    'not_found_in_trash'=> esc_html__('No follow term logs found in Trash','wpwfp'),
						    'parent_item_colon' => '',
						    'menu_name' 		=> esc_html__('Follow Term Logs','wpwfp'),
						);
	$follow_term_logs_args = array(
						    'labels' 				=> $follow_term_logs_labels,
						    'public' 				=> false,
						    'query_var' 			=> false,
						    'rewrite' 				=> false,
						    'capability_type' 		=> WPW_FP_TERM_LOGS_POST_TYPE,
						    'hierarchical' 			=> false,
						    'supports' 				=> array( 'title' )
					 	);
	
	//register follow terms logs post type
	register_post_type( WPW_FP_TERM_LOGS_POST_TYPE, $follow_term_logs_args );
	
	//follow author - post type
	$follow_author_labels = array(
						    'name'				=> esc_html__('Follow Author','wpwfp'),
						    'singular_name' 	=> esc_html__('Follow Author','wpwfp'),
						    'add_new' 			=> esc_html__('Add New','wpwfp'),
						    'add_new_item' 		=> esc_html__('Add New Follow Author','wpwfp'),
						    'edit_item' 		=> esc_html__('Edit Follow Author','wpwfp'),
						    'new_item' 			=> esc_html__('New Follow Author','wpwfp'),
						    'all_items' 		=> esc_html__('All Follow Authors','wpwfp'),
						    'view_item' 		=> esc_html__('View Follow Author','wpwfp'),
						    'search_items' 		=> esc_html__('Search Follow Author','wpwfp'),
						    'not_found' 		=> esc_html__('No follow authors found','wpwfp'),
						    'not_found_in_trash'=> esc_html__('No follow authors found in Trash','wpwfp'),
						    'parent_item_colon' => '',
						    'menu_name' 		=> esc_html__('Follow Authors','wpwfp'),
						);
	$follow_author_args = array(
						    'labels' 				=> $follow_term_labels,
						    'public' 				=> false,
						    'query_var' 			=> false,
						    'rewrite' 				=> false,
						    'capability_type' 		=> WPW_FP_AUTHOR_POST_TYPE,
						    'hierarchical' 			=> false,
						    'supports' 				=> array( 'title' )
						); 
	
	//register follow author post type
	register_post_type( WPW_FP_AUTHOR_POST_TYPE, $follow_author_args );
	
	//follow Author logs - post type
	$follow_author_logs_labels = array(
						    'name'				=> esc_html__('Follow Author Logs','wpwfp'),
						    'singular_name' 	=> esc_html__('Follow Author Log','wpwfp'),
						    'add_new' 			=> esc_html__('Add New','wpwfp'),
						    'add_new_item' 		=> esc_html__('Add New Follow Author Log','wpwfp'),
						    'edit_item' 		=> esc_html__('Edit Follow Author Log','wpwfp'),
						    'new_item' 			=> esc_html__('New Follow Author Log','wpwfp'),
						    'all_items' 		=> esc_html__('All Follow Author Logs','wpwfp'),
						    'view_item' 		=> esc_html__('View Follow Author Log','wpwfp'),
						    'search_items' 		=> esc_html__('Search Follow Author Log','wpwfp'),
						    'not_found' 		=> esc_html__('No follow author logs found','wpwfp'),
						    'not_found_in_trash'=> esc_html__('No follow author logs found in Trash','wpwfp'),
						    'parent_item_colon' => '',
						    'menu_name' 		=> esc_html__('Follow Author Logs','wpwfp'),
						);
	$follow_author_logs_args = array(
						    'labels' 				=> $follow_author_logs_labels,
						    'public' 				=> false,
						    'query_var' 			=> false,
						    'rewrite' 				=> false,
						    'capability_type' 		=> WPW_FP_AUTHOR_LOGS_POST_TYPE,
						    'hierarchical' 			=> false,
						    'supports' 				=> array( 'title' )
					 	);
	
	//register follow authors logs post type
	register_post_type( WPW_FP_AUTHOR_LOGS_POST_TYPE, $follow_author_logs_args );
	
}
//register custom post type
add_action( 'init', 'wpw_fp_register_post_types', 100 ); // we need to keep priority 100, because we need to execute this init action after all other init action called.
?>